/*
 * IRC - Internet Relay Chat, ircd/m_z.c
 * Copyright (C) 2003-2005 Progs
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: m_z.c,v 1.6 2005/02/01 13:55:45 progs Exp $
 */

/*
 * m_functions execute protocol messages on this server:
 *
 *    cptr    is always NON-NULL, pointing to a *LOCAL* client
 *            structure (with an open socket connected!). This
 *            identifies the physical socket where the message
 *            originated (or which caused the m_function to be
 *            executed--some m_functions may call others...).
 *
 *    sptr    is the source of the message, defined by the
 *            prefix part of the message if present. If not
 *            or prefix not found, then sptr==cptr.
 *
 *            (!IsServer(cptr)) => (cptr == sptr), because
 *            prefixes are taken *only* from servers...
 *
 *            (IsServer(cptr))
 *                    (sptr == cptr) => the message didn't
 *                    have the prefix.
 *
 *                    (sptr != cptr && IsServer(sptr) means
 *                    the prefix specified servername. (?)
 *
 *                    (sptr != cptr && !IsServer(sptr) means
 *                    that message originated from a remote
 *                    user (not local).
 *
 *            combining
 *
 *            (!IsServer(sptr)) means that, sptr can safely
 *            taken as defining the target structure of the
 *            message in this server.
 *
 *    *Always* true (if 'parse' and others are working correct):
 *
 *    1)      sptr->from == cptr  (note: cptr->from == cptr)
 *
 *    2)      MyConnect(sptr) <=> sptr == cptr (e.g. sptr
 *            *cannot* be a local connection, unless it's
 *            actually cptr!). [MyConnect(x) should probably
 *            be defined as (x == x->from) --msa ]
 *
 *    parc    number of variable parameter strings (if zero,
 *            parv is allowed to be NULL)
 *
 *    parv    a NULL terminated list of parameter pointers,
 *
 *                    parv[0], sender (prefix string), if not present
 *                            this points to an empty string.
 *                    parv[1]...parv[parc-1]
 *                            pointers to additional parameters
 *                    parv[parc] == NULL, *always*
 *
 *            note:   it is guaranteed that parv[0]..parv[parc-1] are all
 *                    non-NULL pointers.
 */
#include "config.h"

#include "handlers.h"
#include "channel.h"
#include "client.h"
#include "hash.h"
#include "ircd.h"
#include "ircd_reply.h"
#include "ircd_string.h"
#include "ircd_features.h"
#include "ircd_relay.h"
#include "msg.h"
#include "numeric.h"
#include "numnicks.h"
#include "s_conf.h"
#include "s_debug.h"
#include "s_user.h"
#include "send.h"

#include <assert.h>
#include <stdlib.h>
#include <string.h>

int m_z(struct Client* cptr, struct Client* sptr, int parc, char* parv[])
{
  struct	Client* acptr;
  char		parmstring[512] = {0}, mask[NICKLEN + HOSTLEN + 1];
  int		i, n;

  if (parc < 2 || EmptyString(parv[1]))
	return send_reply(sptr, ERR_NOTEXTTOSEND);

  acptr = FindUser(feature_str(FEAT_SERVICE_NICK));
  /* check si le nick SERVICE_NICK exist et si il est bien sur SERVICE_NAME */
  if (!acptr || strcasecmp(cli_name(cli_user(acptr)->server), feature_str(FEAT_SERVICE_NAME)))
    return send_reply(sptr, ERR_SERVICESDOWN, feature_str(FEAT_SERVICE_NICK));

  for(i=1,n=0;i<parc && n < 512;i++)
  {
    while(*parv[i] && n < 512) parmstring[n++] = *parv[i]++;
    parmstring[n++] = ' ';
  }
  parmstring[n] = 0;
  strcpy(mask,feature_str(FEAT_SERVICE_NICK));
  strcat(mask, "@");
  strcat(mask,feature_str(FEAT_SERVICE_NAME));
  relay_directed_message(sptr, mask, strchr(mask, '@'), parmstring);
  return 0;
}
